@@ -215,6 +215,12 @@ public class WebSSOProviderImpl implements SAMLWebSSOProvider {
         requestInfo.setDomainName(domainName);
         requestInfo.setLoginPath(httpRequest.getParameter("loginPath"));
         requestInfo.setClientID(httpRequest.getParameter("client"));
+        String uriFragment = httpRequest.getParameter(SAMLLoginTools.PARAM_URI_FRAGMENT);
+        if (uriFragment == null) {
+            // fallback for deprecated parameter name
+            uriFragment = httpRequest.getParameter("hash");
+        }
+        requestInfo.setUriFragment(uriFragment);
 
         String relayState = stateManagement.addAuthnRequestInfo(requestInfo, AUTHN_REQUEST_TIMEOUT, TimeUnit.MILLISECONDS);
         try {
@@ -292,6 +298,11 @@ public class WebSSOProviderImpl implements SAMLWebSSOProvider {
                 redirectLocation.setParameter(LoginFields.CLIENT_PARAM, clientID);
             }
 
+            String uriFragment = requestInfo.getUriFragment();
+            if (uriFragment != null) {
+                redirectLocation.setParameter(SAMLLoginTools.PARAM_URI_FRAGMENT, uriFragment);
+            }
+
             Tools.disableCaching(httpResponse);
             httpResponse.sendRedirect(redirectLocation.build().toString());
         } catch (URISyntaxException e) {
@@ -324,7 +335,7 @@ public class WebSSOProviderImpl implements SAMLWebSSOProvider {
              */
             if (config.isAllowUnsolicitedResponses()) {
                 String relayState = httpRequest.getParameter("RelayState");
-                if (relayState == null) {
+                if (Strings.isEmpty(relayState)) {
                     // use DefaultAuthnRequestInfo if no RelayState is set
                     String domainName = getDomainName(httpRequest);
                     requestInfo = new DefaultAuthnRequestInfo();
@@ -370,7 +381,7 @@ public class WebSSOProviderImpl implements SAMLWebSSOProvider {
     private void tryExchangeAssertionForOAuthToken(Assertion assertion, AuthenticationInfo authInfo) throws OXException {
         try {
             OAuthAccessTokenService service = services.getService(OAuthAccessTokenService.class);
-            if(service==null){
+            if (service==null){
                 LOG.debug("OAuthAccessTokenService is missing. Unable to exchange the assertion {} for an oauth token pair.", assertion.getID());
                 return;
             }
@@ -385,7 +396,7 @@ public class WebSSOProviderImpl implements SAMLWebSSOProvider {
             LOG.debug("Trying to exchange the assertion {} for an oauth token pair...", assertion.getID());
             OAuthAccessToken token = service.getAccessToken(OAuthGrantType.SAML, b64Assertion, authInfo.getUserId(), authInfo.getContextId(), null);
             authInfo.getProperties().put(SAMLSessionParameters.ACCESS_TOKEN, token.getAccessToken());
-            if(token.getRefreshToken()!=null){
+            if (token.getRefreshToken()!=null){
                 authInfo.getProperties().put(SAMLSessionParameters.REFRESH_TOKEN, token.getRefreshToken());
                 LOG.debug("Successfully exchanged the assertion {} for an oauth access and refresh token.", assertion.getID());
             } else {
