@@ -49,11 +49,8 @@
 
 package com.openexchange.saml.osgi;
 
-import com.openexchange.config.ConfigurationService;
-import com.openexchange.osgi.HousekeepingActivator;
-import com.openexchange.saml.impl.DefaultConfig;
-import com.openexchange.saml.impl.SAMLConfigRegistryImpl;
-import com.openexchange.saml.spi.SAMLConfigRegistry;
+import org.osgi.framework.BundleActivator;
+import org.osgi.framework.BundleContext;
 
 /**
  * OSGi activator for com.openexchange.saml.
@@ -62,34 +59,22 @@ import com.openexchange.saml.spi.SAMLConfigRegistry;
  * @author <a href="mailto:steffen.templin@open-xchange.com">Steffen Templin</a>
  * @since v7.6.1
  */
-public class SAMLActivator extends HousekeepingActivator {
+public class SAMLActivator implements BundleActivator {
 
     private SAMLFeature samlFeature;
 
     @Override
-    protected Class<?>[] getNeededServices() {
-        return new Class[] { ConfigurationService.class };
-    }
-
-    @Override
-    protected synchronized void startBundle() throws Exception {
+    public synchronized void start(BundleContext context) throws Exception {
         samlFeature = new SAMLFeature(context);
         samlFeature.open();
-
-        SAMLConfigRegistryImpl configRegistry = SAMLConfigRegistryImpl.getInstance();
-        DefaultConfig config = DefaultConfig.init(getService(ConfigurationService.class));
-        configRegistry.registerSAMLConfig(SAMLConfigRegistryImpl.DEFAULT_KEY, config);
-        registerService(SAMLConfigRegistry.class, configRegistry);
-
     }
 
     @Override
-    protected synchronized void stopBundle() throws Exception {
+    public synchronized void stop(BundleContext context) throws Exception {
         if (samlFeature != null) {
             samlFeature.close();
             samlFeature = null;
         }
-        super.stopBundle();
     }
 
 }
