@@ -76,6 +76,7 @@ import org.slf4j.LoggerFactory;
 import com.hazelcast.core.HazelcastInstance;
 import com.openexchange.ajax.AJAXServlet;
 import com.openexchange.ajax.login.LoginRequestHandler;
+import com.openexchange.config.ConfigurationService;
 import com.openexchange.dispatcher.DispatcherPrefixService;
 import com.openexchange.exception.OXException;
 import com.openexchange.java.ConcurrentList;
@@ -87,6 +88,7 @@ import com.openexchange.saml.http.AssertionConsumerService;
 import com.openexchange.saml.http.InitService;
 import com.openexchange.saml.http.MetadataService;
 import com.openexchange.saml.http.SingleLogoutService;
+import com.openexchange.saml.impl.DefaultConfig;
 import com.openexchange.saml.impl.DefaultLoginConfigurationLookup;
 import com.openexchange.saml.impl.LoginConfigurationLookup;
 import com.openexchange.saml.impl.SAMLLoginRequestHandler;
@@ -128,13 +130,18 @@ public final class SAMLBackendRegistry extends ServiceTracker<SAMLBackend, SAMLB
      */
     public SAMLBackendRegistry(BundleContext context, ServiceLookup services) throws BundleException {
         super(context, SAMLBackend.class, null);
-        this.openSAML = initOpenSAML();
-        this.loginConfigurationLookup = new DefaultLoginConfigurationLookup();
-        this.services = services;
-        this.backends = new ConcurrentList<SAMLBackend>();
-        this.backendServlets = new ConcurrentHashMap<SAMLBackend, Stack<String>>();
-        this.backendServiceRegistrations = new ConcurrentHashMap<SAMLBackend, Stack<ServiceRegistration<?>>>();
-        this.hzStateManagement = new HzStateManagement(services.getService(HazelcastInstance.class));
+        try {
+            DefaultConfig.init(services.getServiceSafe(ConfigurationService.class));
+            this.openSAML = initOpenSAML();
+            this.loginConfigurationLookup = new DefaultLoginConfigurationLookup();
+            this.services = services;
+            this.backends = new ConcurrentList<SAMLBackend>();
+            this.backendServlets = new ConcurrentHashMap<SAMLBackend, Stack<String>>();
+            this.backendServiceRegistrations = new ConcurrentHashMap<SAMLBackend, Stack<ServiceRegistration<?>>>();
+            this.hzStateManagement = new HzStateManagement(services.getService(HazelcastInstance.class));
+        } catch (OXException e) {
+            throw new BundleException("Initialization of SAML back-end registry failed.", e);
+        }
     }
 
     @Override
