@@ -66,15 +66,22 @@ import com.openexchange.groupware.notify.hostname.HostnameService;
 import com.openexchange.hazelcast.serialization.CustomPortableFactory;
 import com.openexchange.osgi.DependentServiceStarter;
 import com.openexchange.saml.SAMLProperties;
-import com.openexchange.saml.impl.SAMLConfigRegistryImpl;
+import com.openexchange.saml.impl.DefaultConfig;
+import com.openexchange.saml.impl.DefaultLoginConfigurationLookup;
+import com.openexchange.saml.impl.VeryDangerousSAMLBackend;
 import com.openexchange.saml.impl.SAMLSessionInspector;
+import com.openexchange.saml.impl.SAMLSessionSsoProvider;
+import com.openexchange.saml.impl.SAMLSessionStorageParameterNamesProvider;
 import com.openexchange.saml.impl.hz.PortableAuthnRequestInfoFactory;
 import com.openexchange.saml.impl.hz.PortableLogoutRequestInfoFactory;
 import com.openexchange.saml.oauth.service.OAuthAccessTokenService;
+import com.openexchange.saml.spi.SAMLBackend;
 import com.openexchange.server.ServiceLookup;
+import com.openexchange.session.SessionSsoProvider;
 import com.openexchange.session.inspector.SessionInspectorService;
 import com.openexchange.session.reservation.SessionReservationService;
 import com.openexchange.sessiond.SessiondService;
+import com.openexchange.sessionstorage.SessionStorageParameterNamesProvider;
 import com.openexchange.templating.TemplateService;
 import com.openexchange.user.UserService;
 
@@ -87,8 +94,6 @@ import com.openexchange.user.UserService;
  */
 public class SAMLFeature extends DependentServiceStarter {
 
-    private SAMLBackendRegistry samlBackends;
-
     private static final Logger LOG = LoggerFactory.getLogger(SAMLFeature.class);
 
     private final static Class<?>[] NEEDED_SERVICES = new Class[] {
@@ -109,6 +114,10 @@ public class SAMLFeature extends DependentServiceStarter {
         HostnameService.class
     };
 
+    // -------------------------------------------------------------------------------------------------------------------------------------
+
+    private SAMLBackendRegistry samlBackends;
+
     private final Stack<ServiceRegistration<?>> serviceRegistrations = new Stack<ServiceRegistration<?>>();
 
     private final Stack<String> servlets = new Stack<String>();
@@ -118,7 +127,7 @@ public class SAMLFeature extends DependentServiceStarter {
     }
 
     @Override
-    protected void start(ServiceLookup services) throws Exception {
+    protected synchronized void start(ServiceLookup services) throws Exception {
         ConfigurationService configService = services.getService(ConfigurationService.class);
         boolean enabled = configService.getBoolProperty(SAMLProperties.ENABLED, false);
         if (enabled) {
@@ -127,16 +136,24 @@ public class SAMLFeature extends DependentServiceStarter {
             serviceRegistrations.push(context.registerService(SessionInspectorService.class, new SAMLSessionInspector(sessiondService), null));
             serviceRegistrations.push(context.registerService(CustomPortableFactory.class, new PortableAuthnRequestInfoFactory(), null));
             serviceRegistrations.push(context.registerService(CustomPortableFactory.class, new PortableLogoutRequestInfoFactory(), null));
+            serviceRegistrations.push(context.registerService(SessionStorageParameterNamesProvider.class, new SAMLSessionStorageParameterNamesProvider(), null));
+            serviceRegistrations.push(context.registerService(SessionSsoProvider.class, new SAMLSessionSsoProvider(new DefaultLoginConfigurationLookup(), sessiondService), null));
+
             getSamlBackend(services);
+
+            if (configService.getBoolProperty("com.openexchange.saml.startVeryDangerousDebugBackend", false)) {
+                serviceRegistrations.push(context.registerService(SAMLBackend.class, new VeryDangerousSAMLBackend(services.getService(UserService.class), services.getService(ContextService.class)), null));
+            }
         } else {
             LOG.info("SAML 2.0 support is disabled by configuration. Skipping initialization...");
         }
     }
 
     /**
-     * Helper method to get the SAMLBackendRegistry
-     * @param services the ServiceLookup
-     * @throws BundleException if start fails
+     * Helper method to initialize the <code>SAMLBackendRegistry</code>.
+     *
+     * @param services The service look-up
+     * @throws BundleException If initialization fails
      */
     private void getSamlBackend(ServiceLookup services) throws BundleException {
         SAMLBackendRegistry samlBackends = this.samlBackends;
@@ -148,7 +165,7 @@ public class SAMLFeature extends DependentServiceStarter {
     }
 
     @Override
-    protected void stop(ServiceLookup services) {
+    protected synchronized void stop(ServiceLookup services) {
         LOG.info("Stopping SAML 2.0 support...");
         HttpService httpService = services.getService(HttpService.class);
         while (!servlets.isEmpty()) {
@@ -163,7 +180,7 @@ public class SAMLFeature extends DependentServiceStarter {
             samlBackends.stop();
             samlBackends = null;
         }
-        SAMLConfigRegistryImpl.getInstance().clear();
+        DefaultConfig.release();
     }
 
 }
